package MusicDB::Album;

use XML::Simple;
use XML::ValidWriter;
use XML::Doctype;

use Data::Dumper;

sub new 
{
	my $proto = shift;
	my $class = ref($proto) || $proto;
	my $self = {};
	bless($self,$class);
	return $self;
}

## Export as XML.
sub export_xml
{
	my $self = shift;
	my $filename = shift;
	open(FILE,">$filename") or die "Couldn't create $filename: $!";

	my $dtd = XML::Doctype->new( 'album', SYSTEM_ID => 'albuminfo.dtd');
	my $writer = XML::ValidWriter->new(
		DOCTYPE => $dtd,
		OUTPUT => \*FILE
	);

	$writer->setDataMode(1);
	$writer->xmlDecl;
	$writer->startTag('album', type => $self->{info}->{type}, uuid=> $self->{info}->{uuid} || '');
	$writer->dataElement('title', $self->{info}->{title} || '');
	if($self->{info}->{type} eq 'single-artist') {
		$writer->dataElement('artist', $self->{info}->{artist} || '');
	}
	## We're ripping, so the source is the original CD.
	$writer->dataElement('source', 'original CD');
	if(defined($self->{info}->{genre})) {
		$writer->dataElement('genre', $self->{info}->{genre});
	}
	$writer->startTag('tracklist');
	foreach my $track (@{$self->{info}->{tracks}}) {

		$writer->startTag('track', uuid => $track->{uuid} || '');
		$writer->dataElement('title',$track->{title});
		foreach my $artist (@{$track->{artist}}) {
			$writer->dataElement('artist',$artist->{content},type => $artist->{type});
		}
		$writer->dataElement('track-number', $track->{number} || '');
		$writer->dataElement('length',$track->{len} || 0,units => 'milliseconds');
		$writer->dataElement('filename',$track->{filename});
		$writer->endTag('track');
	}	
	$writer->endTag('tracklist');
	if(defined($self->{info}->{cddbinfo})) {
		$writer->startTag('cddbinfo');
		$writer->dataElement('genre', $self->{info}->{cddbinfo}->{genre});
		$writer->dataElement('id', $self->{info}->{cddbinfo}->{id});
		$writer->endTag('cddbinfo');
	}
	
	if(defined($self->{info}->{barcode})) {
		$writer->dataElement('barcode', $self->{info}->{barcode}->{content}, type => $self->{info}->{barcode}->{type})
	}
	if(defined($self->{info}->{tableofcontents})) {
		$writer->startTag('tableofcontents');
		foreach my $trk (keys %{$self->{info}->{tableofcontents}}) {
			$writer->dataElement('toctrack', '',
				number => $trk, 
				minutes => $self->{info}->{tableofcontents}->{$trk}->{minutes},
				seconds => $self->{info}->{tableofcontents}->{$trk}->{seconds},
				frames => $self->{info}->{tableofcontents}->{$trk}->{frames}
				);
			
		}
		$writer->endTag('tableofcontents');
	}
	if(defined($self->{info}->{toc_data})) {
		$writer->dataElement('toc', $self->{info}->{toc_data});
	}
	$writer->endTag('album');
	$writer->end;
	close(FILE);

}

sub import_xml
{
	my $self = shift;
	my $filename = shift;
	my $ref = XMLin(
			$filename, 
			suppressempty => 1, 
			keyattr => {}, 
			forcearray => [ 'track' ]);
	
	# Fix tracks.
	$ref->{tracks} = $ref->{tracklist}->{track};
	delete($ref->{tracklist}->{track});

	# fix track numbers
	foreach my $track (@{$ref->{tracks}}) {
		$track->{number} = $track->{'track-number'};
		delete($track->{'track-number'});
	}

	# fix artist
	my $foo = $ref->{artist}->{content};
	delete($ref->{artist});
	$ref->{artist} = $foo;
	
	# mangle old format <toc> into new <tableofcontents>
	if($ref->{toc}) {
		$ref->{tableofcontents} = {};
		foreach my $line (split(/\n/,$ref->{toc})) {
			my ($trk,$m,$s,$f) = $line =~ /(\d+)\s+(\d+)\s+(\d+)\s+(\d+)/;
			$ref->{tableofcontents}->{$trk} = {
				minutes => $m,
				seconds => $s,
				frames  => $f
			};
		}
		delete($ref->{toc});
	}

	print Dumper $ref;
	
	$self->{info} = $ref;
	
}

=pod 

=head2 USAGE

  my $album = new MusicDB::Album;
  $album->fetch_from_cddb();  
  $album->manual_edit();
  $album->export('filename.xml');

  my $album2 = new MusicDB::Album;
  $album2->import('filename.xml');
  if(!$album2->title()) {
	$album2->title('Untitled');
  }

  (need get/set for each of the items in the dtd.)

=cut

## Get/set title
sub title
{
}

1;
